#
# Copyright (c) 2013-2014, 2018, 2021-2022 SIOS Technology Corp.
#
# Description: library to parse/display license information
#
# Author: Paul Clements
#
######################## LifeKeeper Linux Change Log #######################
# bug3699    06Sep13 jef Change vAppKeeper to LifeKeeper SSP
# bug7027    06JUN14 Tatsu Subscription license support
# bug7435    18May18 ryo support storage QWK
# PL-7770    28Mar22 mcp Add hana database to kits


package License;
use POSIX (strftime, locale_h);
setlocale( LC_ALL, "C" );

chomp(my $hostid = `lmutil lmhostid -n`);
$hostid =~ s/"//g;
# add internet hostid to list
chomp(my $hostip=`lmutil lmhostid -n -internet`);
$hostip =~ s/INTERNET=(.*)/$1/;
$hostid .= " $hostip";
my %hostids = map { $_ => 1 } split ' ', $hostid; # flex 11.6 lists multiple IDs

our %locales = ( 'ja' => 'Japan', 'zh' => 'China' );
our %kits = (
	'appsuite_clearcase' => 'Clearcase Recovery Kit',
	'appsuite_mqseries' => 'MQ Series Recovery Kit',
	'appsuite_sap' => 'SAP Recovery Kit',
	'database_hana' => 'SAP HANA Recovery Kit',
	'database_db2' => 'DB2 Recovery Kit',
	'database_informix' => 'Informix Recovery Kit',
	'database_listener' => 'Oracle Listener Recovery Kit',
	'database_mysql' => 'MySQL Recovery Kit',
	'database_oracle' => 'Oracle Recovery Kit',
	'database_pgsql' => 'Postgres Recovery Kit',
	'database_sapdb' => 'SAP DB Recovery Kit',
	'database_sybase' => 'Sybase Recovery Kit',
	'gen_hanfs' => 'NFS Recovery Kit (2/2)',
	'gen_nfs' => 'NFS Recovery Kit (1/2)',
	'gen_samba' => 'Samba Recovery Kit',
	'lsrf_xpclx' => 'XP Cluster Extension ARK',
	'mail_postfix' => 'Postfix Recovery Kit',
	'mail_sams' => 'Sendmail SAMS Recovery Kit',
	'mail_sendmail' => 'Sendmail Recovery Kit',
	'scsi_disrec' => 'Multi-Site Cluster',
	'scsi_dmmp' => 'Device Mapper Multipath (1/2)',
	'scsi_dmmpp' => 'Device Mapper Multipath (2/2)',
	'scsi_drbd' => 'DRBD Recovery Kit',
	'scsi_hdlm' => 'HDLM Recovery Kit (1/2)',
	'scsi_hdlmp' => 'HDLM Recovery Kit (2/2)',
	'scsi_lvmvg' => 'LVM Recovery Kit (1/2)',
	'scsi_lvmlv' => 'LVM Recovery Kit (2/2)',
	'scsi_netraid' => 'DataKeeper for Linux ARK',
	'scsi_netstorage' => 'NAS Recovery Kit',
	'scsi_ppath' => 'PowerPath Recovery Kit (1/2)',
	'scsi_ppathp' => 'PowerPath Recovery Kit (2/2)',
	'scsi_sdd' => 'SDD Recovery Kit (1/2)',
	'scsi_sddp' => 'SDD Recovery Kit (2/2)',
	'scsi_sps' => 'SPS Recovery Kit (1/2)',
	'scsi_spsp' => 'SPS Recovery Kit (2/2)',
	'steeleyeVM_VMwareESX' => 'VMware ESX Recovery Kit',
	'webserver_apache' => 'Apache Recovery Kit',
	'quorum_disk' => 'Storage Quorum Witness Kit'
);

# a license file can contain multiple licenses -- return array of hashes,
# where each hash contains information for one license
sub parse_lic_file {
	my $file = shift;
	my @licenses;
	my %lic;
	my $in_sign;

	open(FILE, "<$file") or return undef;
	foreach (<FILE>) {
		s/\\//g;
		@_ = split ' ', $_;
# print "$_[0] $_[1] $_[2] $_[3] $_[4]\n";
		/^FEATURE/ || /^INCREMENT/ and do {
			defined %lic and do {
				my %newlic = %lic;
				push @licenses, \%newlic;
			};
			dbg_print_lic(\%lic);
			undef %lic;
			undef $in_sign;
			$lic{'feat'} = $_[1];
			$lic{'ver'} = $_[3];
			$lic{'exp'} = $_[4];
# print "$_[0] $_[1] $_[2] $_[3] $_[4]\n";
		};
		foreach (@_) {
			/^VENDOR_STRING=/ and $lic{'vendor'} = (split '=', $_)[1];
			/^HOSTID=/ and do {
				/.*=(.*?)$/; # HOSTID=<mac> or HOSTID=INTERNET=<ip>
				$lic{'hostid'} = $1;
				$lic{'hostidtype'} = 'e';
				/INTERNET/ && do { $lic{'hostidtype'} = 'i' };
			};
			/^SN=/ and do {
				/.*=(.*?)$/; # SN=<entitlement_id>
				$lic{'sn'} = $1;
			};
			/^SIGN=/ || $in_sign and do {
				if (!$in_sign) {
					$lic{'sign'} = (split '=', $_)[1];
					$in_sign = 1;
				} else {
					$lic{'sign'} .= $_;
				}
			};
		}
	}
	close(FILE);
	defined %lic and do push @licenses, \%lic;
	return @licenses;
}

sub date_to_epoch {
	my $date = shift; # format:dd-mmm-yyyy or permanent
	
	# if permanent, expire date is set to 31-dec-9999 in order to process simply.
	if ($date eq 'permanent') {
		$date = '31-dec-9999';
	}
	
	chomp(my $epochsecs = `date --date="$date" +'%s'`);
	return $epochsecs;
}

sub get_days {
	my $left_secs = shift;
	my $right_secs = shift;	
	return int(($left_secs + 86400 - $right_secs) / 60 / 60 / 24);
}

sub dbg_print_lic {
	my $lic = shift;
	return if (!exists($ENV{'LIC_DEBUG'}));
	foreach (keys %$lic) {
		print STDERR "$_ = $lic->{$_}\n";
		if ($_ eq 'feat') {
			lic_info($lic);
			printf STDERR "%s License: $lic->{'info'}\n", $lic->{'type'} eq 'e' ? 'Eval' : 'Permanent';
		}
	}
}

sub lic_exp_info {
	my $lic = shift;
	if ($lic->{'exp'} eq 'permanent') {
		return 'permanent';
	}
	my $expsecs = date_to_epoch($lic->{'exp'});
	chomp(my $expdate = strftime "%d %b %Y", localtime($expsecs));
	my $cursecs = time();
	if ($expsecs < $cursecs) {
		$lic->{'err'}++;
		return "$expdate (EXPIRED)";
	} else {
		my $days = get_days($expsecs, $cursecs);
		if ($days <= 3) { $lic->{'warn'}++; }
		return "$expdate ($days days)";
	}
}

sub lic_restricted_info {
	my $lic = shift;
	my $vendor = $lic->{'vendor'};

	if ($vendor =~ /eval:0/) {
		$lic->{'type'} = 'e';
	}
	if ($vendor =~ /steeleyeVM_XenVM/) {
		return "XenServer Protection Suite";
	} elsif ($vendor =~ /steeleye_datarep/) {
		return "DataKeeper for Linux";
	} elsif ($vendor =~ /steeleye_generic/) {
		return "LifeKeeper Core - restricted";
	}
}

sub lic_info {
	my $lic = shift;
	my $feat = $lic->{'feat'};
	my @fields = split '_', $feat;
	my $locale = 1 if (exists ($locales{$fields[1]}));
	my ($prod, $app, $res) = @fields[0, 1, 2];
	my $locale = $locales{$fields[1]};
	if (defined($locale)) {
		$lic->{'extrainfo'} = "Locale: $locale";
		undef $app; # no localized kits;
		undef $res;
	}
	my $type = substr($prod, -1); # 'e' for eval, 'r' for restricted

	$lic->{'type'} = $type;

	if ($type eq 'r') {
		$lic->{'info'} = lic_restricted_info($lic);
	} elsif ($app eq '' && $prod =~ /^lklcs/) {
		$lic->{'info'} = 'LifeKeeper SSP';
	} elsif ($app eq '' && $prod =~ /^lklc/) {
		$lic->{'info'} = 'LifeKeeper for Linux';
	} elsif ($app ne '') {
		$lic->{'info'} = $kits{$app . '_' . $res};
	}
	if ($lic->{'info'} eq '') {
		$lic->{'info'} = "Unknown ($prod $app $res)";
		$lic->{'warn'}++;
	}
	$lic->{'expinfo'} = lic_exp_info($lic);

	# check for errors
	my $lichost = $lic->{'hostid'};
	if ($lichost && $lichost ne 'ANY' && !exists($hostids{$lichost})) {
		my $dispid = hostid_of_type($lic->{'hostidtype'}, $hostid);
		$lic->{'extrainfo'} = "HOSTID mismatch: $lichost ($dispid)";
		$lic->{'err'}++;
	}
	if ($prod =~ /^..w/) {
		$lic->{'extrainfo'} = "Intended for Windows Product";
		$lic->{'err'}++;
	}
}

sub hostid_of_type {
	my $type = shift;
	my $hostid = shift;
	my @list;

	foreach (split ' ', $hostid) {
		if (/^[0-9]*\./) { # IP/INTERNET
			push @list, $_ if ($type eq 'i');
		} else { # MAC/ETHERNET
			push @list, $_ if ($type eq 'e');
		}
	}
	return join ' ', @list;
}

1;
